-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class CALL_INFIX_TIMES
	--
	--   Infix operator : "*".
	--

inherit
	CALL_INFIX
		redefine static_simplify
		end

creation {EIFFEL_PARSER}
	make

creation {AGENT_CREATION}
	with

feature {ANY}
	precedence: INTEGER is 8

	left_brackets: BOOLEAN is False

	frozen operator: STRING is
		do
			Result := as_muls
		end

	
	static_simplify: EXPRESSION is
		local
			ic1, ic2: INTEGER_CONSTANT; i1, i2, i3: INTEGER_64
		do
			target := target.static_simplify
			arguments.static_simplify
			ic1 ?= target
			if ic1 /= Void then
				ic2 ?= arg1
				if ic2 /= Void then
					i1 := ic1.value_memory
					i2 := ic2.value_memory
					i3 := i1 #* i2
					if i2 /= 0 then
						if (i3 // i2) /= i1 then
							error_handler.append(integer_constant_overflow)
							error_handler.add_position(feature_name.start_position)
							error_handler.print_as_fatal_error
						end
					end
					create {INTEGER_CONSTANT} Result.make(i3, feature_name.start_position)
				end
			end
			if Result = Void then
				Result := Current
			end
		end

	compile_to_c (type: TYPE) is
		do
			call_proc_call_c2c(type)
		end

	compile_to_jvm (type: TYPE) is
		do
			call_proc_call_c2jvm(type)
		end

	jvm_branch_if_false (type: TYPE): INTEGER is
		do
			Result := jvm_standard_branch_if_false(type)
		end

	jvm_branch_if_true (type: TYPE): INTEGER is
		do
			Result := jvm_standard_branch_if_true(type)
		end

feature {ANY}
	accept (visitor: CALL_INFIX_TIMES_VISITOR) is
		do
			visitor.visit_call_infix_times(Current)
		end

feature {}
	make (lp: like target; operator_position: POSITION; rp: like arg1) is
		require
			lp /= Void
			not operator_position.is_unknown
			rp /= Void
		do
			target := lp
			create feature_name.infix_name(eiffel_parser.muls_name, operator_position)
			create arguments.make_1(rp)
		ensure
			target = lp
			start_position = operator_position
			arguments.first = rp
		end

end -- class CALL_INFIX_TIMES
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
